#ifndef __ASM_PCI_H
#define __ASM_PCI_H
#ifdef __KERNEL__

#include <linux/types.h>
#include <linux/slab.h>
#include <linux/dma-mapping.h>

#include <asm/io.h>
#include <asm-generic/pci-bridge.h>
#include <asm-generic/pci-dma-compat.h>

#define PCIBIOS_MIN_IO		0x1000
#define PCIBIOS_MIN_MEM		0

/*
 * Set to 1 if the kernel should re-assign all PCI bus numbers
 */
#define pcibios_assign_all_busses() \
	(pci_has_flag(PCI_REASSIGN_ALL_BUS))

/*
 * PCI address space differs from physical memory address space
 */
#define PCI_DMA_BUS_IS_PHYS	(0)

extern int isa_dma_bridge_buggy;

#ifdef CONFIG_PCI
static inline int pci_proc_domain(struct pci_bus *bus)
{
	return 1;
}
#endif  /* CONFIG_PCI */

/* "PCI MMCONFIG %04x [bus %02x-%02x]" */
#define PCI_MMCFG_RESOURCE_NAME_LEN (22 + 4 + 2 + 2)

#define PCI_MMCFG_BUS_OFFSET(bus)      ((bus) << 20)

struct acpi_device;

struct pci_sysdata {
	int		domain;		/* PCI domain */
	int		node;		/* NUMA node */
	struct acpi_device *companion;	/* ACPI companion device */
	void		*iommu;		/* IOMMU private data */
};

struct acpi_pci_root;
struct pci_mmcfg_region;

typedef int (*acpi_mcfg_fixup_t)(struct acpi_pci_root *root,
				 struct pci_mmcfg_region *cfg);

struct pci_mmcfg_region {
	struct list_head list;
	struct resource res;
	int (*read)(struct pci_mmcfg_region *cfg, unsigned int bus,
		    unsigned int devfn, int reg, int len, u32 *value);
	int (*write)(struct pci_mmcfg_region *cfg, unsigned int bus,
		     unsigned int devfn, int reg, int len, u32 value);
	acpi_mcfg_fixup_t fixup;
	void *data;
	u64 address;
	char __iomem *virt;
	u16 segment;
	u8 start_bus;
	u8 end_bus;
	char name[PCI_MMCFG_RESOURCE_NAME_LEN];
};

struct acpi_mcfg_fixup {
	char oem_id[7];
	char oem_table_id[9];
	acpi_mcfg_fixup_t hook;
};

/* Designate a routine to fix up buggy MCFG */
#define DECLARE_ACPI_MCFG_FIXUP(oem_id, table_id, hook)	\
	static const struct acpi_mcfg_fixup __acpi_fixup_##hook __used	\
	__attribute__((__section__(".acpi_fixup_mcfg"), aligned((sizeof(void *))))) \
	= { {oem_id}, {table_id}, hook };

extern struct pci_mmcfg_region *pci_mmconfig_lookup(int segment, int bus);

#endif  /* __KERNEL__ */
#endif  /* __ASM_PCI_H */
